#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <vector>
#include <list>

using namespace std;

#include "vektor.h"
#include "LLL.h"
#include "print.h"
#include "division.h"
#include "buchberger.h"
#include "toric.h"
#include "flip.h"
#include "test.h"
#include "lp.h"
#include "parser.h"
#include "radical.h"

#include "testresult.h"


int numberofvektors=0;
int maxnumberofvektors=0;

int option_latex;
int option_traversalalgorithm;
int option_startingalgorithm;
int option_facetsuperset;
int option_facetsubset;
int option_facetsoplex;
int option_vertexprinting;
int option_detailedprinting;
char *option_filename_comments;
char *option_filename_results;
char *option_filename_list;
char *option_filename_input;
char *option_filename_input_weight;
bool option_radical_split;
bool option_use_extensions;
bool option_use_radical_file;

static char *changeFileExt(const char *name, const char *extension)
{
  int dotpos=-1;
  int i=0;

  while(name[i])
    {
      if(name[i]=='.')dotpos=i;
      i++;
    }
  if(dotpos==-1)
    dotpos=i;

  char *ret=(char*)malloc(strlen(extension)+dotpos+4);
  assert(ret);
  for(i=0;i<dotpos;i++)ret[i]=name[i];
  sprintf(ret+dotpos,"%s",extension);

  return ret;
}

static void printhelp(FILE *f=stdout)
{
  char *help=
"This is CaTS version 2.0 (January 2003),\n"
"program for Computing Toric State polytopes\n"
"by Anders Nedergaard Jensen.\n"
"Homepage: http://daimi.au.dk/~u950710/cats.html\n"
"The Program computes all reduced Gr\\\"obner bases of a toric ideal\n"
"when given a matrix or a single reduced Gr\\\"obner basis\n"
"defining the ideal.\n"
"It uses the simplex algorithm implementation by Birkett Huber\n"
"or the SoPlex linear programming library when linked properly.\n"
"\n"
"Links:\n"
"SoPlex: Roland Wunderling, \"Paralleler und Objekt-orientierter\n"
" Simplex-Algorithmus\" ZIB technical report TR 96-09, Berlin 1996\n"     
" http://www.zib.de/Optimization/Software/Soplex/soplex.php\n"
"TiGERS: Birkett Huber and Rekha R. Thomas \"Computing Gr\\\"obner\n"
" Fans of Toric Ideals\", Experimental Mathematics, Vol 9 (2000),\n"
" No. 3: 321-331\n"
" http://www.math.washington.edu/~thomas/tigers.html\n"
"TOPCOM: Joerg Rambau, \"TOPCOM: Triangulations of Point Configura-\n"
" tions and Oriented Matroids\" ZIB-Report 02-17, Berlin 2002\n"
" http://www.zib.de/rambau/TOPCOM.html\n"
"CaTS: Anders Nedergaard Jensen, \"Computing Gr\\\"obner Fans of\n"
" Toric Ideals\", Master thesis, Department of Mathematical\n"
" Sciences, University of Aarhus, Denmark, June 2002\n"
" http://www.daimi.au.dk/~u950710/speciale.html\n"
"\n"
"OPTIONS:\n"
"\n"
"Traversal algorithm:\n"
"-t0  None\n"
"-t1  Exhaustive Search\n"
"-t2 *Reverse Search\n"
"-t3  Oriented Exhaustive Search\n"
"-t4  Hybrid\n"
"-t5  Exhaustive Search, fixed radical\n"
"-t6  Reverse Search, fixed radical\n"
"-w   Reverse search on each radical specified in the weight file from TOPCOM\n"
"\n"
"Facet detection:\n"
"-fts0  Disable subset test\n"
"-fts1 *Enable subset test\n"
"-ftS0  Disable superset test\n"
"-ftS1 *Enable superset test\n"
"-fs   *SoPlex Library for linear programming\n"
"-fh    Huber\'s simplex algorithm implementation\n"
"\n"
"LaTeX:\n"
"-l0 Disable LaTeX output\n"
"-l1 Enable LaTeX output\n"
"\n"
"Vertex printing:\n"
"-pd Detailed printing of the graph (only on small examples and\n    together with -t1)\n"
"-p0  No vertex printing\n"
"-p1  Print the facet binomials for each cone in TiGERS' format when found\n"
"-p1+ Print each reduced Gr\\\"obner basis stripped when found\n"
"-p2  Print each monomial initial ideal in TiGERS' format when found\n"
"-p2+ Print each monomial initial ideal stripped when found\n"
"-p3  Print each reduced Gr\\\"obner basis in native format when found\n"
"-ps  Split printing, print each radical, together with -p2+ and -w only\n"
"\n"
"Output file name options:\n"
"-oc FILENAME  file for the comments (including the LaTeX \n              and .dvi files) Default: output_comments\n"
"-or FILENAME  file for the results (time, size of graph, \n              floating point status ....) Default: output_results\n"
"-ol FILENAME  file for listing the graph during traversal\n              (see options -p1, -p2 and -p3) Default: output_list\n"
"\n"
"Input file name options:\n"
"-i FILENAME   input file\n"
"-w FILENAME   weight file, containing heights for each radical\n"
"\n"
"File name options:\n"
"-e  Force use of file name extensions (.heights,.comments,.results,.list)\n"
"\n"
"Help:\n"
"-h Print this help\n"
"\n"
;

   fprintf(f,help);

   if(lp_soplexlinked)
     fprintf(f,"SoPlex library is available.\n");
   else
     fprintf(f,"SoPlex library is not available.\n");
}

int main(int argc, char* argv[])
{
  option_traversalalgorithm=2;
  option_startingalgorithm=0;
  option_facetsuperset=1;
  option_facetsubset=1;
  option_facetsoplex=1;
  option_latex=0;
  option_detailedprinting=0;
  option_vertexprinting=0;
  option_filename_comments="output_comments";
  option_filename_results="output_results";
  option_filename_list="output_list";
  option_filename_input=0;
  option_filename_input_weight=0;
  option_radical_split=false;
  option_use_extensions=false;
  option_use_radical_file=false;

  bool ok=true;

  for(int t=1;t<argc;t++)
    {
      char *s=argv[t];
      if(*s=='-')
	{
	  s++;
	  switch(*s)
	    {
	    case 'i':
	      if(s[1])ok=false;
	      option_filename_input=argv[++t];
	      break;
	    case 'o':
	      s++;
	      if(s[1])ok=false;
	      switch(*s)
		{
		case 'c':
		  option_filename_comments=argv[++t];
		  break;
		case 'r':
		  option_filename_results=argv[++t];
		  break;
		case 'l':
		  option_filename_list=argv[++t];
		  break;
		default:
		  ok=false;
		}
	      break;
	    case 'p':
	      s++;
	      switch(*s)
		{
		case 'd':
		  option_detailedprinting=1;
		  break;
		case '0':
		  option_vertexprinting=0;
		  break;
		case '1':
		  option_vertexprinting=1;
		  if(s[1]=='+')option_vertexprinting+=128;
		  break;
		case '2':
		  option_vertexprinting=2;
		  if(s[1]=='+')option_vertexprinting+=128;
		  break;
		case '3':
		  option_vertexprinting=3;
		  break;
		case 's':
		  option_radical_split=true;
		  break;
		default:
		  ok=false;
		}
	      break;
	    case 'l':
	      s++;
	      switch(*s)
		{
		case '0':
		  option_latex=0;
		  break;
		case '1':
		  option_latex=1;
		  break;
		default:
		  ok=false;
		}
	      break;
	    case 't':
	      s++;
	      switch(*s)
		{
		case '0':
		  option_traversalalgorithm=0;
		  break;
		case '1':
		  option_traversalalgorithm=1;
		  break;
		case '2':
		  option_traversalalgorithm=2;
		  break;
		case '3':
		  option_traversalalgorithm=3;
		  break;
		case '4':
		  option_traversalalgorithm=4;
		  break;
		case '5':
		  option_traversalalgorithm=5;
		  break;
		case '6':
		  option_traversalalgorithm=6;
		  break;
		default:
		  ok=false;
		}
	      break;
	    case 'e':
	      option_use_extensions=true;
	      break;
	    case 'w':
	      if(s[1])ok=false;
	      option_use_radical_file=true;
	      if(t+1>=argc || argv[t+1][0]!='-')
		option_filename_input_weight=argv[++t];
	      break;
	    case 'f':
	      s++;
	      switch(*s)
		{
		case 't':
		  s++;
		  switch(*s)
		    {
		    case 's':
		      option_facetsubset=(s[1]=='1');
		      break;
		    case 'S':
		      option_facetsuperset=(s[1]=='1');
		      break;
		    default:
		      ok=false;
		    }
		  break;
		case 'h':
		  option_facetsoplex=0;
		  break;
		case 's':
		  option_facetsoplex=1;
		  break;
		default:
		  ok=false;
		}
	      break;
	    default:
	      ok=false;
	    }
	}
      else
	ok=false;
    }

  if(!ok)
    {
      printhelp();
      exit(1);
    }

  if(option_filename_input_weight==0)
    if(option_filename_input)
      option_filename_input_weight=changeFileExt(option_filename_input,".heights");
  if(option_filename_input)
    {
      if(option_use_extensions)
	{
	  option_filename_comments=changeFileExt(option_filename_input,".comments");
	  option_filename_results=changeFileExt(option_filename_input,".results");
	  option_filename_list=changeFileExt(option_filename_input,".list");
	  option_filename_input_weight=changeFileExt(option_filename_input,".heights");
	}

      stdin=freopen(option_filename_input,"r",stdin);
      assert(stdin);
    }

  fprintf(stderr,"Using files:\n");
  if(option_filename_input)fprintf(stderr,"%s\n",option_filename_input);
  fprintf(stderr,"%s\n",option_filename_comments);
  fprintf(stderr,"%s\n",option_filename_results);
  fprintf(stderr,"%s\n",option_filename_list);
  fprintf(stderr,"%s\n",option_filename_input_weight);
  fprintf(stderr,"\n");
 
  flip_listfile=fopen(option_filename_list,"w");
  assert(flip_listfile);
  if(!option_radical_split)
    if(option_vertexprinting>=128)
      fprintf(flip_listfile,"{");

  //  if(latex==2)gfanprinton=true;

  
  if(option_latex)
    {
      static char buf[256];
      sprintf(buf,"%s.tex",option_filename_comments);
      print_beginLaTeX(buf);
    }
  else
    {
      stdout=freopen(option_filename_comments,"w",stdout);
    }

  print_alfabeta();

  basis_i B;//=readmatrix();
  gbasis inputgbasis;
  int inputtype=parseinput(B,inputgbasis);

  print_timebegin();

  gbasis g;

  switch(inputtype)
    {
    case 1://matrix
      g=toric1(B);
      printf("\n");
      break;
    case 2://gbasis
      g=inputgbasis;
      gbasisminimal(&g);
      gbasisautoreduction(&g);
      if(option_traversalalgorithm!=1 && option_traversalalgorithm!=5)
	{
	  printf("\n\nWe must make sure that the input basis is a reduced Gr\\\"obner basis with respect to the current termorder when using reverse search or ordered exhaustive search.\n");
	  fprintf(stderr,"\n\nWe must make sure that the input basis is a reduced Gr\\\"obner basis with respect to the current termorder when using reverse search or ordered exhaustive search.\n");
	  printf("\nRunning Buchberger's algorithm...\n");
	  buchberger(g);
	  gbasisminimal(&g);
	  gbasisautoreduction(&g);
	  print(g,stderr);
	}
      break;
    }


  

  //  printt(g);

  fflush(stdout);

  //  vektor_i::TERMORDER=LEXICOGRAPHIC;
  //  vektor_i::setcheapest(0);

  if(!option_use_radical_file)
    {
      switch(option_traversalalgorithm)
	{
	case 1:
	  printf("\n\nComputing the Gr\\\"obner fan using exhaustive search...\n\n");
	  computegrobnerfan(g);
	  break;
	case 2:
	  printf("\n\nComputing the Gr\\\"obner fan using the reverse search method...\n\n");
	  computegrobnerfan_rs(g);
	  break;
	case 3:
	  printf("\n\nComputing the Gr\\\"obner fan using a faster exhaustive search...\n\n");
	  useordered=true;
	  computegrobnerfan(g);
	  break;
	case 4:
	  printf("\n\nComputing the Gr\\\"obner fan using a hybrid algorithm...\n\n");
	  computegrobnerfan_hb(g);
	  break;
	case 5:
	  printf("\n\nExhaustive, fixed radical algorithm...\n\n");
	  computegrobnerfan_fixedradical(g);
	  break;
	case 6:
	  printf("\n\nReverse, fixed radical algorithm...\n\n");
	  computegrobnerfan_rs_fixedradical(g);
	  break;
	}
      printf("\n\nThe state polytope consists of %i vertices and %i edges.\n\n",computegrobnerfan_size,computegrobnerfan_nedges);  
    }
  else
    {
      bool radicalSplit=option_radical_split;
      int i=0;
      int totalNumber=0;
      assert(option_filename_input_weight);
      RadicalParser parser(option_filename_input_weight);
      while(parser.parseRadical())
	{
	  //set weight vector
	  vektor_i::weightenable=true;
	  vektor_i::weight_double_enable=true;
	  vektor_i::weight_double=parser.getWeightVector();
	  assert(vektor_i::weight_double.size()==g.begin()->size());
	  //compute fixed radical
	  buchberger(g);
	  gbasisminimal(&g);
	  gbasisautoreduction(&g);	  

	  if(radicalSplit)
	    {
	      fprintf(flip_listfile,"Radical %i: ",i);
	      gbasis radical=initialradical(g);
	      print_tigers_in_strip(radical,0,flip_listfile);
	      fprintf(flip_listfile,"\n");
	      fprintf(flip_listfile,"Number of known symmetries: %i\n",parser.getNumberOfSymmetries());
	      fprintf(flip_listfile,"{");
	    }
	  foreachvertex_n=0;
	  computegrobnerfan_rs_fixedradical(g);
	  //	    computegrobnerfan_fixedradical(g);
	  if(radicalSplit)
	    {
	      fprintf(flip_listfile,"}\n");
	      fprintf(flip_listfile,"Number of initial ideals with this radical: %i\n\n",computegrobnerfan_size);
	    }
	  fprintf(stderr,"Radical no. %i: %i vertices, %i symmetries\n",i,computegrobnerfan_size,parser.getNumberOfSymmetries());
	  totalNumber+=computegrobnerfan_size*parser.getNumberOfSymmetries();
	  print(vektor_i::weight_double,stderr);
	  fprintf(stderr,"Total number of initial ideals so far: %i\n",totalNumber); 
	  i++;
	}
      computegrobnerfan_size=totalNumber;
      computegrobnerfan_nedges=-1;
    }

  testresult t=testresult_new();
  t.time=print_gettime();
  t.size_gfan=computegrobnerfan_size;
  t.nedges_gfan=computegrobnerfan_nedges;
  if(inputtype==1)
    {
      t.dim_d=B[0].size();
      t.dim_n=B.size();
      t.kerdim=toric1_kerdim;
    }
  t.floatingpointstatus=lp_status;
  testresult_write(t,option_filename_results);
  
  if(option_latex)
    {
      print_endLaTeX();
    }
  else
    {
      fclose(stdout);
    }
  //  fprintf(stderr,"number of vektors currently allocated:%i\n",numberofvektors);
  //  fprintf(stderr,"max number of vektors allocated:%i\n",maxnumberofvektors);
  
  if(lp_status==0)
    {
      fprintf(stderr,"No floating point problems detected.\n");
    }
  else
    {
      fprintf(stderr,"Floating point troubles! Result is not reliable.\n");
    }
  
  if(!option_radical_split)
    if(option_vertexprinting>=128)
      fprintf(flip_listfile,"}\n");
  fclose(flip_listfile);

  return 0;
}




