#include "parser.h"
#include <stdio.h>
#include <assert.h>
#include "print.h"

static int nextchar;

static int get(FILE *f)
{
  // skip initial comments
  int c;

  do
    {
      c=getc(f);
      if(c=='%')while(c!='\n')c=getc(f);
    }
  while(c=='\n');

  return c;
}

static void getnext()
{
  do
    {
      nextchar=getc(stdin);
      
      assert(nextchar!=EOF);
    }
  while(nextchar== '\n' || nextchar== '\t' || nextchar == ' ');
}

static vektor_i parsemonomial(int dim)
{
  vektor_i v(dim);

  while(nextchar>='a' && nextchar<='z')
    {
      int n=1;
      int index=nextchar-'a';
      getnext();
      if(nextchar=='^')
	{
	  scanf("%i",&n);
	  getnext();
	}
      assert(index>=0);
      assert(index<dim);
      v[index]+=n;
      if(nextchar=='*')getnext();
    }

  print(v,stderr);
  return v;
}

static vektor_i parsebinomial(int dim)
{
  vektor_i a=parsemonomial(dim);
  assert(nextchar=='-');
  getnext();
  vektor_i b=parsemonomial(dim);
  return a-b;
}

static gbasis parsegbasis(int dim)
{
  gbasis g;

  //  getnext();
  assert(nextchar=='{');
  getnext();
  while(nextchar!='}')
    {
      g.push_back(parsebinomial(dim));
      assert(nextchar==','|| nextchar == '}');
      if(nextchar==',')getnext();
    }
  return g;
}

int parseinput(basis_i &matrix_, gbasis &basis)
{
  basis_i B;
  int c;
  char matrix[40*40*4];

  fprintf(stderr,"type in ascii representation:");
  c=getc(stdin);
  ungetc(c,stdin);
  if(c=='{' || c=='[' || c=='(')
    {
      int i=0;
      int b=0;

      do
	{
	  assert(i<40*40*4-1);
	  matrix[i++]=getc(stdin);

	  if(matrix[i-1]=='[' && b==0)matrix[i-1]='{';
	  if(matrix[i-1]=='[' && b==1)matrix[i-1]='(';

	  if(matrix[i-1]=='{')b++;
	  if(matrix[i-1]=='(')b++;
	  if(matrix[i-1]=='[')b++;
	  if(matrix[i-1]==']')b--;
	  if(matrix[i-1]==')')b--;
	  if(matrix[i-1]=='}')b--;

	  if(matrix[i-1]==']' && b==1)matrix[i-1]=')';
	  if(matrix[i-1]==']' && b==0)matrix[i-1]='}';
	}
      while(matrix[i-1]!='}' || b!=0);
      //      fprintf(stderr,"%s\n",matrix);
      matrix[i]=0;
      B=tobasis_i(matrix);
    }
  else
    {
      //      c=get(stdin);
      getnext();
      c=nextchar;
      switch(c)
	{
	  case 'M':
	    {
	      int ncolumns;
	      int nrows;

	      while(c!='{')c=getc(stdin);
	      fscanf(stdin,"%i %i",&nrows,&ncolumns);
	      while(c!=':')c=getc(stdin);
	      for(int i=0;i<ncolumns;i++)
		{
		  vektor_i v(nrows);
		  B.push_back(v);
		}
	      for(int j=0;j<nrows;j++)
		for(basis_i::iterator k=B.begin();k!=B.end();k++)
		  {
		    int t;
		    fscanf(stdin,"%i",&t);
		    k->v[j]=t;
		  }
	    }
	    break;
	case 'R':
	  {
	    gbasis g;
	    int dim;
	    getnext();
	    while(nextchar!=':')getnext();
	    scanf("%i",&dim);
	    getnext();
	    while(nextchar!=':')getnext();
	    while(nextchar!='{')getnext();
	    g=parsegbasis(dim);

	    fprintf(stderr,"\nParsed Gr\\\"obner basis:\n");
	    print_tigers_strip(g,0,stderr);

	    basis=g;
	    return 2;
	  }
	  break;
	default:
	  fprintf(stderr,"Parse Error!\n");
	  assert(0);
	}
      /*M:
	{3 6 :
	2 1 0 1 0 0
	0 1 2 1 1 0
	0 0 0 0 1 2
	} */
    }
  fprintf(stderr,"\nInput Matrix Columns:\n");
  print(B,stderr);
  matrix_=B;

  return 1;
}


//--------------------------------------------------------
// Radical File Parser
//--------------------------------------------------------

RadicalParser::RadicalParser(const char *filename)
{
  f=fopen(filename,"r");
  assert(f);
  mode=1;
}


RadicalParser::~RadicalParser()
{
  fclose(f);
  f=0;
}

bool RadicalParser::readWeightVector()
{
  char line[4096];
  int c;
  while((c=fgetc(f))!='(' && c!='T' && c!=EOF);
  ungetc(c,f);
  if(c!='(')return false;
  fgets(line,4096,f);
  vektor_f weight_vector(line);
  currentWeightVector=weight_vector;
  return true;
}

bool RadicalParser::parseRadical()
{
  if(!readWeightVector())return false;
  if(mode==0)
    {
      currentNumberOfSymmetries=1;
      return true;
    }

  currentNumberOfSymmetries=1;
  while(readWeightVector())currentNumberOfSymmetries++;

  int c;
  while((c=fgetc(f))!='(' && c!='T' && c!=EOF);

  assert(c=='T');
  while((c=fgetc(f))!='\n' && c!=EOF);

  assert(c=='\n');
  return true;
}


vektor<double> RadicalParser::getWeightVector()
{
  return currentWeightVector;
}


int RadicalParser::getNumberOfSymmetries()
{
  return currentNumberOfSymmetries;
}
