#include "print.h"
#include <stdlib.h>
#include <time.h>
#include "flip.h"

#define LETTER 'a'

static int LaTeX=0;
static int alfabeta=0;

static char *filename;

void print_alfabeta()
{
  alfabeta=1;
}

void print_math(FILE* f)
{
  if(LaTeX)fprintf(f," $ ");
}

void print_beginLaTeX(char *filename_, FILE* f)
{
  filename=filename_;
  f=freopen(filename,"w",f);

  fprintf(f,"\\documentclass[12pt,a4paper,dvips]{article}"
	  "\\usepackage[english]{babel}"
	  "\\usepackage{t1enc}"
	  "\\usepackage{a4}"
	  "\\usepackage{epsfig}"
	  "\\usepackage{amsfonts}"
	  "\\usepackage{latexsym}"
	  "\\begin{document}\n");
  LaTeX=1;
}

void print_endLaTeX(FILE *f)
{
  fprintf(f,"\\end{document}");
  LaTeX=0;

  fclose(f);

  char buf[256];
  sprintf(buf,"latex %s>/dev/null",filename);
  system(buf);
}

void print(const vektor_f &v, FILE* f)
{
#define FLOATREPFORMAT "%5.2f"
	int s=v.size();
	int i=0;

	fprintf(f,"(");
	if(i<s)fprintf(f,FLOATREPFORMAT,float(v[i]));
	for(i++;i<s;i++)fprintf(f,","FLOATREPFORMAT,float(v[i]));
	fprintf(f,")\n");
}

void print(const vektor_i &v, FILE* f)
{
  if(LaTeX)fprintf(f,"\n");
	int s=v.size();
	int i=0;

	fprintf(f,"(");
	if(i<s)fprintf(f,"%i",(v[i]));
	for(i++;i<s;i++)fprintf(f,",%i",(v[i]));
	fprintf(f,")\n");
  if(LaTeX)fprintf(f,"\n");
}

void printmatrix(basis_i &b, FILE* f)
{
  int col=b.size();
  if(col==0)return;
  int row=b[0].size();
  
  //  fprintf(f,"$\\left[$\n");
  fprintf(f,"$\\left( \\begin{array}{rrrrrrrrrrrrrrrrrrr}\n");
  for(int i=0;i<row;i++)
    {
      int j;
      for(j=0;j<col-1;j++)
	printf("%i&",b[j][i]);
      printf("%i \\\\\n",b[j][i]);
    }
  fprintf(f,"\\end{array}\\right )$\n");
  //  fprintf(f,"$\\right]$\n");
}

void printmatrixtransposed(basis_i &b, FILE* f)
{
  int row=b.size();
  if(row==0)return;
  int col=b[0].size();
  
  //  fprintf(f,"$\\left[$\n");
  fprintf(f,"$\\left( \\begin{array}{rrrrrrrrrrrrrrrrrrr}\n");
  for(int i=0;i<row;i++)
    {
      int j;
      for(j=0;j<col-1;j++)
	printf("%i&",b[i][j]);
      printf("%i \\\\\n",b[i][j]);
    }
  fprintf(f,"\\end{array}\\right )$\n");
  //  fprintf(f,"$\\right]$\n");
}

void printmatrix(gbasis &b, FILE* f)
{
  int col=b.begin()->size();
  if(col==0)return;
  //  int row=b.begin()->size();
  
  //  fprintf(f,"$\\left[$\n");
  fprintf(f,"\\begin{tabular}[t]{rrrrrrrrrrrr}\n");
  for(gbasis::iterator i=b.begin();i!=b.end();i++)
    {
      int j;
      for(j=0;j<col-1;j++)
	printf("%i & ",(*i)[j]);
      printf("%i \\\\\n",(*i)[j]);
    }
  fprintf(f,"\\end{tabular}\n");
  //  fprintf(f,"$\\right]$\n");
}

void printmono(const vektor_f &v, int sign, FILE *f)
{
  int flag=0;
  for(int i=0;i<v.size();i++)if(v[i]*sign>0)
    {
      fprintf(f,"%c",i+LETTER);
      if(int(v[i]*sign)!=1)fprintf(f,"%i",int(v[i]*sign));
      flag=1;
    }
  if(!flag)fprintf(f,"1");
}

void printmono(const vektor_i &v, int sign, FILE *f)
{
  int flag=0;
  for(int i=0;i<v.size();i++)if(v[i]*sign>0)
    {
      if(alfabeta)
	fprintf(f,"%c",i+LETTER);
      else
	{
	  fprintf(f,"x_{%i}",i+1);
	}
      if(int(v[i]*sign)!=1)
	{
	  if(LaTeX)
	    fprintf(f,"^{%i}",int(v[i]*sign));
	  else
	    fprintf(f,"%i",int(v[i]*sign));
	}
      flag=1;
    }
  if(!flag)fprintf(f,"1");
}

void printt(const vektor_f &v, FILE *f)
{
  printmono(v,1,f);
  fprintf(f,"-");
  printmono(v,-1,f);
  fprintf(f,"\n");
}

void printt(const vektor_i &v, FILE *f)
{
  print_math(f);
  printmono(v,1,f);
  fprintf(f,"-");
  printmono(v,-1,f);
  print_math(f);
  fprintf(f,"\n\n");
}

void printt_(const vektor_f &v, FILE *f)
{
  printmono(v,1,f);
  fprintf(f,"-");
  printmono(v,-1,f);
}



void print(double a, FILE* f){fprintf(f,FLOATREPFORMAT,a);}
void print(int a, FILE* f){fprintf(f,"%i",a);}

void print(basis_f &b, FILE* f)
{
	for(int i=0;i<b.size();i++)print(b[i],f);
}

void printt(basis_f &b, FILE* f)
{
	for(int i=0;i<b.size();i++)printt(b[i],f);
}

void print(basis_i &b, FILE* f)
{
	for(int i=0;i<b.size();i++)print(b[i],f);
}

void printt(basis_i &b, FILE* f)
{
	for(int i=0;i<b.size();i++)printt(b[i],f);
}

void printt(const gbasis &b, FILE *f)
{
  for(gbasis::const_iterator i=b.begin();i!=b.end();i++)printt(*i,f);
}

void printinitial(const gbasis &b, FILE *f)
{
  fprintf(f," $ \\langle$ \n");
  for(gbasis::const_iterator i=b.begin();i!=b.end();i++)
    {
      if(i!=b.begin())fprintf(f,", ");
      fprintf(f," $ ");
      printmono(*i,1,f);
      fprintf(f," $ ");
    }
  fprintf(f," $\\rangle $ \n");
}

static bool ismember(const vektor_i &p, const gbasis &b)
{
  for(gbasis::const_iterator i=b.begin();i!=b.end();i++)
    {
      vektor_i a=*i-p;
      if(a.iszero())return true;
    }
  return false;
}

void printt2(const gbasis &b, FILE *f)
{
  gbasis irr=irredundant(b);

  for(gbasis::const_iterator i=b.begin();i!=b.end();i++)
    {
      if(!ismember(*i,irr))
	{
	  printt(*i);
	}
      else
	{
	  printf("\\fbox{");
	  printt(*i);
	  printf("}\n\n");
	}
    }
}

void print(const gbasis &b, FILE *f)
{
  for(gbasis::const_iterator i=b.begin();i!=b.end();i++)print(*i,f);
  if(LaTeX)fprintf(f,"\n\n");
}


void printmono_tigers(const vektor_i &v, int sign, FILE *f)
{
  int flag=0;
  for(int i=0;i<v.size();i++)if(v[i]*sign>0)
    {
      if(flag)fprintf(f,"*");
      if(alfabeta)
 fprintf(f,"%c",i+LETTER);
      else
 {
   fprintf(f,"x_{%i}",i+1);
 }
      if(int(v[i]*sign)!=1)
 {
     fprintf(f,"^%i",int(v[i]*sign));
 }
      flag=1;
    }
  if(!flag)fprintf(f,"1");
}

void printt_tigers(const vektor_i &v, FILE *f)
{
  printmono_tigers(v,1,f);
  fprintf(f,"-");
  printmono_tigers(v,-1,f);
}

void print_tigers(const gbasis &g, int num, FILE *f)
{
  gbasis facets=irredundant(g);
  int binomials=g.size();
  int degree=0;

  for(gbasis::const_iterator i=g.begin();i!=g.end();i++)
    {
      int curdeg=0;
      for(int t=0;t<i->size();t++)
 if(i->v[t]>0)curdeg+=i->v[t];
      if(curdeg>degree)degree=curdeg;
    }

  fprintf(f,"Vtx: %i (%i facets/%i binomials/degree %i)\n",num,facets.size(),binomials,degree);

  fprintf(f,"{");
  for(gbasis::iterator i=facets.begin();i!=facets.end();i++)
    {
      if(i!=facets.begin())fprintf(f,", ");
      fprintf(f,"# ");
      printt_tigers(*i,f);
    }
  fprintf(f,"}\n");
}

void print_tigers_in(const gbasis &g, int num, FILE *f)
{
  gbasis facets=irredundant(g);
  int binomials=g.size();
  int degree=0;

  for(gbasis::const_iterator i=g.begin();i!=g.end();i++)
    {
      int curdeg=0;
      for(int t=0;t<i->size();t++)
 if(i->v[t]>0)curdeg+=i->v[t];
      if(curdeg>degree)degree=curdeg;
    }

  fprintf(f,"Vtx: %i (%i facets/%i binomials/degree %i)\n",num,facets.size(),binomials,degree);

  fprintf(f,"    Initial ideal:{");
  for(gbasis::const_iterator i=g.begin();i!=g.end();i++)
    {
      if(i!=g.begin())fprintf(f,", ");
      printmono_tigers(*i,1,f);
    }
  fprintf(f,"}\n");

  fprintf(f,"  Facet Binomials:{");
  for(gbasis::iterator i=facets.begin();i!=facets.end();i++)
    {
      if(i!=facets.begin())fprintf(f,",");
      fprintf(f,"# ");
      printt_tigers(*i,f);
    }
  fprintf(f,"}\n");
}

void print_tigers_strip(const gbasis &g, int num, FILE *f)
{
  fprintf(f,"{");
  for(gbasis::const_iterator i=g.begin();i!=g.end();i++)
    {
      if(i!=g.begin())fprintf(f,", ");
      printt_tigers(*i,f);
    }
  fprintf(f,"}");
}

void print_tigers_in_strip(const gbasis &g, int num, FILE *f)
{

  fprintf(f,"{");
  for(gbasis::const_iterator i=g.begin();i!=g.end();i++)
    {
      if(i!=g.begin())fprintf(f,", ");
      printmono_tigers(*i,1,f);
    }
  fprintf(f,"}");
}













static int timestart;
static int timestartsec;

void print_timebegin()
{
  timestartsec=time(0);
  timestart=clock();
  //  printf("Starting clock!\n\n");
}

void print_time()
{
  printf("Time: %8f ms\n\n",((clock()-timestart)*1000.0)/CLOCKS_PER_SEC);
}

int print_gettime()
{
  int t2=time(0);
  if((t2-timestartsec)>(1000))return (t2-timestartsec)*1000;
  return (int)((((double)clock()-(double)timestart))*((1000.0)/CLOCKS_PER_SEC));
}

void print_timeend(FILE *f)
{
  fprintf(f,"%8f\n",((clock()-timestart)*1000.0)/CLOCKS_PER_SEC);
  printf("Time: %8f ms\n\n",((clock()-timestart)*1000.0)/CLOCKS_PER_SEC);
}



